@@ -30,41 +30,41 @@ class UserAuthHelper:
     # one of the best summaries of relevant fields: https://medium.com/@darutk/understanding-id-token-5f83f50fa02e
     def get_user_auth_context_from_event(self, event):
 
-        print("event type is: {} ".format(type(event)))
+        logger.info("event type is: {} ".format(type(event)))
 
         if event is None:
-            print("event is none")
+            logger.info("event is none")
             return None
 
 
         requestContext = event.get("requestContext")
         if requestContext is None:
-            print("request context is none")
+            logger.warning("request context is none")
             return None
 
         authorizer = requestContext.get("authorizer")
         if authorizer is None:
-            print("authorizer is none")
+            logger.warning("authorizer is none")
             return None
 
         claims = authorizer.get("claims")
         if claims is None:
-            print("claims is none")
+            logger.warning("claims is none")
             return None
 
         email = claims.get("email")
         if ((not isinstance(email, str)) or (email == "")):
-            print("email none, invalid type, or empty")
+            logger.warning("email none, invalid type, or empty")
             return None
 
         sub = claims.get("sub")
         if ((not isinstance(sub, str)) or (sub == "")):
-            print("sub none, invalid type, or empty")
+            logger.warning("sub none, invalid type, or empty")
             return None
 
         is_email_verified = claims.get("email_verified")
         if is_email_verified is None:
-            print("email is verified is none")
+            logger.info("email is verified is none")
             #remove early exit, see note below
             #return None
 
@@ -72,11 +72,11 @@ class UserAuthHelper:
         is_email_verified_bool = False
         if ( isinstance(is_email_verified, bool) and (is_email_verified) ):
             #specifically boolean type and true
-            print("email verified found as boolean true")
+            logger.info("email verified found as boolean true")
             is_email_verified_bool = True
         elif (is_email_verified == "true"):
             #specifically a string containing true
-            print("email verified found as string 'true'")
+            logger.info("email verified found as string 'true'")
             is_email_verified_bool = True
 
         if not is_email_verified_bool:
@@ -88,7 +88,7 @@ class UserAuthHelper:
             # google : https://stackoverflow.com/a/30357942/7632432
             # amazon : can't confirm without email as currently configured:
             #           https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-settings-email-phone-verification.html
-            print("strict mode would not pass login")
+            logger.info("strict mode would not pass login due to verified flag")
             pass
 
 
@@ -102,7 +102,7 @@ class UserAuthHelper:
     #
     #  returns a valid properly_user_id or None if there was an unauthorizied condition
 
-    def find_or_update_verified_user(self, user_auth: dict, requested_properly_user_id: str):
+    def find_or_update_verified_user(self, user_auth: dict, requested_properly_user_id: str, role: str = None):
 
         auth_id = user_auth.get('auth_id')
         auth_email = user_auth.get('auth_email')
@@ -111,6 +111,10 @@ class UserAuthHelper:
         if (actual_properly_user_id):
             logger.info("Existing Verified:  properly user id: {}  auth id: {}".format(actual_properly_user_id,
                                                                                        auth_id))
+
+            if role != None:
+                self.add_role_for_user(actual_properly_user_id, role)
+
             return actual_properly_user_id
 
 
@@ -124,16 +128,56 @@ class UserAuthHelper:
             #use the matching id
             id_to_update = actual_properly_user_id
 
-
         actual_properly_user_id = self.create_or_update_if_unverified(id_to_update, auth_id, auth_email)
         if not actual_properly_user_id:
             logger.error("Failed to update properly user id: {}  with user auth: {}. ".format(actual_properly_user_id,
                                                                                        user_auth))
             #will occur when something is invalid about the target record to update (some other user claimed, or different email etc.)
             return None
+
+        if role != None:
+            self.add_role_for_user(actual_properly_user_id, role)
+
         return actual_properly_user_id
 
 
+    def add_role_for_user(self, properly_user_id: str, role: str):
+        if role == None:
+            raise ValueError('Must provide a role to add')
+
+        user_key = {'id': properly_user_id}
+
+        seconds_epoch_now = int(time.time())
+
+        update_expression = 'set updatedAt = :updatedAt, ' \
+                            '#r = list_append(if_not_exists(#r, :empty_list), :role_list)'
+
+        expression_attributes = {':updatedAt':seconds_epoch_now,
+                                 ':empty_list' : [],
+                                 ':role_list' : [role],
+                                 ':role_str' : role}
+
+        expression_attribute_names = {'#r' : 'roles'} # for reserved keyword
+
+        condition_expression = 'NOT contains (#r, :role_str)'
+
+        try:
+            response = self.user_table.update_item(
+                    Key=user_key,
+                    UpdateExpression=update_expression,
+                    ExpressionAttributeNames=expression_attribute_names,
+                    ExpressionAttributeValues=expression_attributes,
+                    ConditionExpression = condition_expression
+                )
+        except ClientError as e:
+            # one possible expected 'error' - role already exists.  ignore and continue
+            if e.response['Error']['Code'] == 'ConditionalCheckFailedException':
+                logger.info(f"Attempt to add existing role {role} - skipping")
+                return None
+            logger.error("exception traceback: {}".format(traceback.format_exc()))
+
+            raise
+
 
     def get_properly_id_from_user_auth_id(self, auth_id: str):
 
@@ -214,10 +258,10 @@ class UserAuthHelper:
         # and if the unverfiedEmail either doesn't exist or is the same value (can't verify unverfied accounts with different email)
       #  condition_expression = '( attribute_not_exists(authCognitoId) OR (authCognitoId = :authCognitoId) ) ' \
       #                         ' AND (attribute_not_exists(unverifiedEmail) OR (unverifiedEmail = :email) )'
-
-        condition_expression = '(  authIdCognito = :authIdCognito OR attribute_not_exists(authIdCognito)  ) ' \
-                                 ' AND (unverifiedEmail = :email OR attribute_not_exists(unverifiedEmail) ) '
-
+        # HACK TEMP: allow updating authIdCognito to a new value since we're switching user pools
+#         condition_expression = '(  authIdCognito = :authIdCognito OR attribute_not_exists(authIdCognito)  ) ' \
+#                                  ' AND (unverifiedEmail = :email OR attribute_not_exists(unverifiedEmail) ) '
+        condition_expression = '(unverifiedEmail = :email OR attribute_not_exists(unverifiedEmail)'
         try:
 
             response = self.user_table.update_item(
@@ -258,14 +302,14 @@ class UserAuthHelper:
         if (user_dict_dynamo is None):
             return False  # not in a role if the user doesn't exist
 
-        if (role_id == UserAuthHelper.BASIC_ROLE_ID):
+        if role_id == UserAuthHelper.BASIC_ROLE_ID:
             return True  # if the user exists they are automatically in the basic role
 
         user_dict = DynamoData.to_dict(user_dict_dynamo)
 
         roles_array = user_dict.get(UserSchema.ROLES)
 
-        if (roles_array is None):
+        if roles_array is None:
             return False
 
         has_role_flag = role_id in roles_array
